import React, {
  Component,
  PropTypes,
} from 'react';
import {
  View,
  ScrollView,
  Dimensions,
} from 'react-native';
import moment from 'moment';
import Week from './Week';
import Day from './Day';
import styles from './style';

const SCREEN_WIDTH = Dimensions.get('window').width;
const VIEW_INDEX = 1;

class CalenderContent extends Component {
    static propTypes = {
      weekStart: PropTypes.number,
      startDate: PropTypes.string,
      endDate: PropTypes.string,
      selectedDate: PropTypes.arrayOf(PropTypes.string),
      onDateClick: PropTypes.func,
      removeClippedSubviews: PropTypes.bool, // ScrollView
      renderDay: PropTypes.func,
      scrollEnabled: PropTypes.bool,
      style: PropTypes.shape({
        container: View.propTypes.style,
        week: View.propTypes.style,
      }),
    };

    static defaultProps = {
      startDate: moment().format('YYYY/MM/DD'),
      endDate: null,
      width: SCREEN_WIDTH,
      removeClippedSubviews: true,
      onDateClick: null,
      weekStart: 1,
      renderDay: null,
      scrollEnabled: true,
      selectedDate: null,
      style: null,
    };
    constructor(props) {
      super(props);
      this.months = [];
      this.state = {
        currentMoment: moment(this.props.startDate).startOf('month'), // 当前要改变的state状态
        selectedMoment: moment(this.props.selectedDate[0]), // 选中的state状态
      };
    }

    getDayProps(renderIndex, befourOffset, currentMoment) {
      const dayIndex = renderIndex - befourOffset;
      const selectedMoment = this.state.selectedMoment;
      const thisMoment = moment(currentMoment).add(dayIndex, 'day'); // 补齐offset的空缺
      const title = thisMoment.format('D') === moment(thisMoment).startOf('month').format('D') ? `${thisMoment.format('M')}月` : thisMoment.format('D');
      const isMonthStart = thisMoment.format('D') === moment(thisMoment).startOf('month').format('D');
      const disabled = moment(thisMoment).isBetween(this.props.startDate, this.props.endDate);
      return {
        key: renderIndex,
        title,
        isMonthStart, // 是否是每月的第一天
        style: {},
        date: moment(thisMoment).format('YYYY/MM/DD'), // 回调函数的date
        onPress: () => {
          this.selectDate(thisMoment);
          if (this.props.onDateClick) {
            this.props.onDateClick(thisMoment.format());
          }
        },
        isSelected: selectedMoment.isSame(thisMoment),
        isToday: moment().format('YYYY/MM/DD') === thisMoment.format('YYYY/MM/DD'),
        disabled,
      };
    }

    /* 生成dataSource */
    getMonthes(currentMoment) {
      if (this.months.length === 0 && this.props.scrollEnabled) {
        this.fromCurrentMonth(currentMoment);
        return this.months;
      } else if (this.props.scrollEnabled) {
        this.fromNextMonth(currentMoment);
        return this.months;
      }
      return [moment(currentMoment)];
    }

    fromCurrentMonth(currentMoment) {
      // 第一次初始渲染的时候 1月 2月 3月 foucs
      for (let i = 0; i <= 2; i += 1) {
        const nextMoment = moment(currentMoment).add(i, 'month');
        if (moment(nextMoment).isBefore(this.props.endDate)) this.months.push(nextMoment);
      }
      return this.months;
    }

    fromNextMonth(currentMoment) {
      const startMoment = moment(this.props.startDate).startOf('month');
      // 第一次初始渲染后点击touch 避免重新渲染为 12月 1月 2月
      if (moment(currentMoment).isSame(startMoment)) return this.months;
      this.months = [];
      for (let i = -VIEW_INDEX; i <= VIEW_INDEX; i += VIEW_INDEX) {
        this.months.push(moment(currentMoment).add(i, 'month'));
      }
      return this.months;
    }

    scrollToItem(itemIndex) {
      const scrollToX = itemIndex * SCREEN_WIDTH;
      if (this.props.scrollEnabled && this.calendar) {
        this.calendar.scrollTo({ y: 0, x: scrollToX, animated: false });
      }
    }

    scrollEnded(event) {
      let i;
      const startDate = moment(this.props.startDate).startOf('month');
      const endDate = moment(this.props.endDate).startOf('month');
      const currentMoment = this.state.currentMoment;
      const postion = event.nativeEvent.contentOffset.x;
      const next = postion / SCREEN_WIDTH;
      if (moment(startDate).isSame(currentMoment)) {
        // 初始化时 1 2 3  focus = 1  next=1时跳到2月
        i = next;
      } else if (next !== 1) {
        // 1 2 3 focus = 2 next=0 1月  next=2 3月
        i = next === 0 ? -VIEW_INDEX : VIEW_INDEX;
      }
      const newMoment = moment(currentMoment).add(i, 'month');
      const isBetween = moment(newMoment).isBetween(startDate, endDate);
      if (isBetween) {
        this.setState({ currentMoment: newMoment });
        this.scrollToItem(VIEW_INDEX);
      }
    }

    selectDate(date) {
      this.setState({ selectedMoment: date });
    }

    renderCalendarView(currentMoment) {
      let renderIndex = 0; // 起始渲染位置
      let weekDate = []; // 保存一周的数据
      const weekStart = this.props.weekStart; // 从星期几开始
      const weekRows = []; // 保存每周的数据
      const currStartWeek = currentMoment.isoWeekday() - weekStart; // 每月第一天是从星期几开始做偏移
      const befourOffset = (currStartWeek + 7) % 7; // 偏移之前空几天
      for (renderIndex; renderIndex < 42; renderIndex += 1) {
        const element = this.renderDay(this.getDayProps(renderIndex, befourOffset, currentMoment));
        weekDate.push(element);
        if (renderIndex % 7 === 6) {
          weekRows.push(<Week style={this.props.style} key={`${renderIndex}-${currentMoment}`}>{weekDate}</Week>);
          weekDate = [];
        }
      }
      return <View style={{ width: SCREEN_WIDTH }} key={`${currentMoment}`}>{weekRows}</View>;
    }


    renderDay(parameter) {
      if (this.props.renderDay) {
        return this.props.renderDay(parameter);
      }
      return <Day {...parameter} />;
    }

    render() {
      const { style } = this.props;
      const calendarDates = this.getMonthes(this.state.currentMoment);
      return (
        <View style={[styles.container, style.container]}>
          <ScrollView
            ref={(calendar) => { this.calendar = calendar; }}
            horizontal
            scrollEnabled
            pagingEnabled
            removeClippedSubviews
            scrollEventThrottle={1000} // 每1000S被调用1次
            showsHorizontalScrollIndicator={false}
            automaticallyAdjustContentInsets={false}
            onMomentumScrollEnd={event => this.scrollEnded(event)}
          >
            {calendarDates.map(month => this.renderCalendarView(month))}
          </ScrollView>
        </View>
      );
    }
}
export default CalenderContent;
