import React, {
  Component, PropTypes, ViewStylePropTypes,
} from 'react';

import {
  Animated, Text, View, Dimensions,
} from 'react-native';
import styles from './style';
import Touch from '../Touch';

const DEVICE_WIDTH = Dimensions.get('window').width;
const propTypes = {
  tabs: PropTypes.arrayOf(PropTypes.any),
  goToScene: PropTypes.func,
  activeTextStyle: Text.propTypes.style,
  defaultTextStyle: Text.propTypes.style,
  tabBarStyle: ViewStylePropTypes || (View.propTypes && View.propTypes.style),
  tabUnderLineStyle: ViewStylePropTypes || (View.propTypes && View.propTypes.style), // 下划线背景
  activeTab: PropTypes.number,
};
const defaultProps = {
  tabs: [],
  goToScene: null,
  activeTextStyle: styles.activeTextStyle,
  tabBarPosition: null,
  defaultTextStyle: styles.tabDeafultText,
  tabUnderLineStyle: null,
  tabBarStyle: null,
  activeTab: null,
};

class TabBar extends Component {
  constructor(props) {
    super(props);
    const {
      tabs,
      tabUnderLineStyle,
    } = this.props;
    const itemWidth = DEVICE_WIDTH / tabs.length;
    this.state = {
      tabWidth: 0,
      offset: new Animated.Value(0),
      prevActive: 0,
      marginLeft: (itemWidth - tabUnderLineStyle.width) / 2,
    };
  }

  componentWillUpdate(nextProps) {
    this.Animated(nextProps.activeTab);
  }

  onPress = (key, onPressHandler) => {
    onPressHandler(key);
    this.Animated(key);
  }

  Animated(index) {
    const len = this.props.tabs.length;
    Animated.spring(
      this.state.offset,
      {
        toValue: index * (DEVICE_WIDTH / len),
        duration: 100,
      },
    ).start();
  }

  renderTab = (tabLabel, key, onPressHandler) => {
    const {
      activeTextStyle,
      defaultTextStyle,
      activeTab,
    } = this.props;

    return (<Touch
      style={[{ flex: 1 }]}
      key={tabLabel}
      accessible
      accessibilityLabel={tabLabel}
      accessibilityTraits="button"
      onPress={() => this.onPress(key, onPressHandler)}
    >
      <View style={styles.item}>
        <View style={styles.tabBackground}>
          <Text style={activeTab === key ? activeTextStyle : defaultTextStyle}>{tabLabel}</Text>
        </View>
      </View>
    </Touch>);
  }

  render() {
    const {
      tabs,
      goToScene,
      tabBarStyle,
      tabUnderLineStyle,
    } = this.props;
    const tabWidth = DEVICE_WIDTH / tabs.length;
    return (
      <View>
        <View style={[styles.defaultTabBarStyle, tabBarStyle]}>
          {tabs.map((tabLabel, key) => this.renderTab(tabLabel, key, goToScene))}
        </View>
        <Animated.View style={[{ width: tabWidth }, styles.showUnderline,
          { transform: [{ translateX: this.state.offset }] }]}
        >
          <View style={[tabUnderLineStyle, { marginLeft: this.state.marginLeft }]} />
        </Animated.View>
      </View>
    );
  }
}

TabBar.propTypes = propTypes;
TabBar.defaultProps = defaultProps;

export default TabBar;
